@@ -1,3 +1,4 @@
+/* -*- c-basic-offset: 2; indent-tabs-mode: nil -*- */
 /*
  * Copyright (C) 2000 Bjrn Englund <d4bjorn@dtek.chalmers.se>,
  *                    Hkan Hjort <d95hjort@dtek.chalmers.se>
@@ -18,10 +19,19 @@
  * USA
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#if defined(HAVE_INTTYPES_H)
+#include <inttypes.h>
+#elif defined(HAVE_STDINT_H)
+#include <stdint.h>
+#endif
+
+#include <dvdread/dvd_reader.h>
 #include <dvdread/ifo_print.h>
 
 static char *program_name;
@@ -48,7 +58,9 @@ int main(int argc, char *argv[])
   }
 
   ifoPrint( dvd, atoi( argv[ 2 ] ) );
+  DVDClose(dvd);
 
+  DVDFinish(); //to keep memory checkers from complaining 
   return 0;  
 }
 
