@@ -1,3 +1,4 @@
+/* -*- c-basic-offset: 2; indent-tabs-mode: nil -*- */
 /*
  * Copyright (C) 2001 Billy Biggs <vektor@dumbterm.net>,
  *                    Hkan Hjort <d95hjort@dtek.chalmers.se>
@@ -17,8 +18,16 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 
+#if defined(HAVE_INTTYPES_H)
+#include <inttypes.h>
+#elif defined(HAVE_STDINT_H)
+#include <stdint.h>
+#endif
+
 #include <dvdread/dvd_reader.h>
 #include <dvdread/ifo_types.h>
 #include <dvdread/ifo_read.h>
@@ -97,6 +106,7 @@ int main( int argc, char **argv )
 
     ifoClose( ifo_file );
     DVDClose( dvd );
+    DVDFinish();
     return 0;
 }
 
